@@ -1,18 +1,14 @@
 package MooX::Struct::Util;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.014';
+our $VERSION   = '0.017';
 
 use strict;
 use warnings;
-use Sub::Install qw(install_sub);
+use Exporter::Tiny ();
 
-sub import
-{
-	my $class  = shift;
-	my $caller = caller;
-	install_sub { code => $_, from => $class, into => $caller } for @_;
-}
+our @ISA    = qw(Exporter::Tiny);
+our @EXPORT = qw(lazy_default);
 
 sub lazy_default (&)
 {
@@ -69,7 +65,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2012-2013, 2017 by Toby Inkster.
+This software is copyright (c) 2012-2013, 2017-2018 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
